// Copyright (C) 2012 MicroStrain, Inc.

/*********************
*	console_commands.h
*	console_commands.h holds the implementations for commands operating
*	on the console window of the application's dialog box.
*********************/

#include "2400_sdk.h"
#include "stdafx.h"

#include "console_commands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*	strCmd_info is a hash that holds all the strings associated with each command.  When the user
*	selects a command in the console, the index of that selected item is used and is referenced in
*	strCmd_info to show the string that explains the command in the command information area of the
*	demo applications.	*/
CMapPosToCString strCmd_info;

/*	crCmd_color is the list of colors for outputting to the console window.  When information is
*	output to the screen, the drawing function looks up the corresponding color based on its
*	position in the console window and sets the text color to that index's color.	*/
CMapPosToColor crCmd_color;

/*	CCListBox pointer is used to allow classes throughout the program to
*	write to the console window	*/
CCListBox *pcliConsole;

/************************
*	set_console()
*	set_console() sets the static global pcliConsole pointer to the value given
*	by cliConsole.  This value is then used for other functions to print commands
*	to the console.  This function should only be called once at the beginning of
*	the application's execution as the console window isn't destroyed until the
*	program quits.
*	PRE:	cliConsole is a legitimate CCListBox pointer pointing to the application's
*				console window
*	POST:	pcliConsole now points to the application's console window
************************/
void set_console(/* in */ CCListBox *cliConsole)
{
	pcliConsole = cliConsole;
}

/************************
*	console_write()
*	console_write() writes a string, strMsg, to the console window.
*	PRE:	strMsg is a CString object holding the information to be written
*	POST:	if successful, strMsg has been written to the window and the index
*				of the the string's location in the console has been returned
*			otherwise, -1 has been returned
************************/
int console_write(/* in */ LPCTSTR lpszMsg)
{
	if (pcliConsole == NULL)
		return -1;
	
	int iPos = pcliConsole->AddString(lpszMsg);
	if (iPos == LB_ERR) {
		MessageBox(AfxGetMainWnd()->GetSafeHwnd(), "An error occurred while appending to the console.", "Error", MB_APPLMODAL | MB_OK | MB_ICONERROR);
		iPos = -1;
	}
	else if (iPos == LB_ERRSPACE) {
		MessageBox(AfxGetMainWnd()->GetSafeHwnd(), "There is no longer enough memory to append to the log.  Please, clear it before continuing.", "Error", MB_APPLMODAL | MB_OK | MB_ICONERROR);
		iPos = -1;
	}

	pcliConsole->SetCurSel(iPos);

	return iPos;
}

/************************
*	console_write_command()
*	console_write_command() writes an unsigned character array, ucCmd, to the 
*	screen as a hexidecimal command.  i.e. if we want to output a read command
*	it will appear as 0x03 0x00 0x04 0x00 0x02 to read node 4, EEPROM location 2.
*	PRE:	ucCmd is a legitimate unsigned character array of length dwLength.
*				This array is not null terminated
*			dwLength is the length of the unsigned character array
*	POST:	if successful, ucCmd has been written to the console and the index
*				of the the string's location in the console has been returned
*			otherwise, -1 has been returned
************************/
int console_write_command(/* in */ unsigned char *ucCmd,
							/* in */ DWORD dwLength)
{
	if (pcliConsole == NULL)
		return -1;
	int iPos = 0;
	CString strFinal, strFormat;
	for (DWORD i = 0; i < dwLength; i++) {
		strFormat.Format("%#04x ", ucCmd[i]);
		strFinal += strFormat;
		if (i > 17 && i % 18 == 0) {		//	wrap to the next line so we don't have to horizontally scroll
			iPos = console_write(strFinal);
			strFinal = "";
		}
	}
	if (strFinal != "")	//	make sure the result wasn't just output in the last loop
		iPos = console_write(strFinal);
	return iPos;
}

